#include "cursors.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#define CURSORS 4

#include "cursors/normal_cursor"
#include "cursors/normal_cursor_mask"
#include "cursors/working_cursor_1"
#include "cursors/working_cursor_2"
#include "cursors/working_cursor_3"
#include "cursors/working_cursor_4"
#include "cursors/working_cursor_mask"

static Cursor working_cur[CURSORS];
static Cursor normal_cur;
static unsigned char *bitmaps[CURSORS];
static int current_cur = 0;

#define ADD_CURSOR(bitmap) { \
	source = XCreateBitmapFromData(display, root_window, (char*)bitmap##_bits, bitmap##_width, bitmap##_height); \
	working_cur[i++] = XCreatePixmapCursor(display, source, mask, &fore, &back, 1, 1); \
	XFreePixmap(display, source); \
}

int XDAFCreateCursors(Display * display)
{
  XColor fore, back;
  Pixmap source, mask;
  int i = 0;
  Window root_window;
  Colormap colormap;

  root_window = DefaultRootWindow(display);
  colormap = DefaultColormap(display, DefaultScreen(display));

  XParseColor(display, colormap, "white", &fore);
  XParseColor(display, colormap, "black", &back);

  normal_cur = XCreateFontCursor(display, XC_left_ptr);
  mask =
    XCreateBitmapFromData(display, root_window,
			  (char *) working_cursor_mask_bits,
			  working_cursor_mask_width,
			  working_cursor_mask_height);

  ADD_CURSOR(working_cursor_1);
  ADD_CURSOR(working_cursor_2);
  ADD_CURSOR(working_cursor_3);
  ADD_CURSOR(working_cursor_4);

  XFreePixmap(display, mask);

  return CURSORS;		/* we loaded all the cursors */
}

void XDAFAnimateCursor(Display * display,
		       const char *color,
		       Boolean wheel)
{
  int count;
  XColor fore, back, white;
  Colormap colormap;

  if (color)
  {
    colormap = DefaultColormap(display, DefaultScreen(display));
    XParseColor(display, colormap, color, &fore);
    XParseColor(display, colormap, "black", &back);
    XParseColor(display, colormap, "white", &white);
  }

  if (!wheel)
  {
    if (XGrabPointer
	(display, DefaultRootWindow(display), False, PointerMotionMask,
	 GrabModeSync, GrabModeSync, None, normal_cur,
	 CurrentTime) == GrabSuccess)
    {
      if (color)
	XRecolorCursor(display, normal_cur, &back, &fore);
      XFlush(display);
      usleep(400000);

      XRecolorCursor(display, normal_cur, &back, &white);
      XFlush(display);
      XUngrabPointer(display, CurrentTime);
    }
  }
  else
    for (count = 0; count < 8; count++)
      if (XGrabPointer
	  (display, DefaultRootWindow(display), False, PointerMotionMask,
	   GrabModeSync, GrabModeSync, None, working_cur[current_cur],
	   CurrentTime) == GrabSuccess)
      {
	current_cur = (current_cur + 1) % CURSORS;
	if (color)
	  XRecolorCursor(display, working_cur[current_cur], &fore, &back);
	XFlush(display);
	usleep(50000);
	XUngrabPointer(display, CurrentTime);
      }
}

void XDAFRestoreCursor(Display * display)
{
  XFlush(display);
}

void XDAFUnloadCursors(Display * display)
{

}

static char rcsid[] = "$Id: cursors.c,v 1.8 2002/11/07 23:10:19 ezix Exp $";
