#include <stdio.h>
#include <string.h>

#if defined(hpux)

#include <sys/pstat.h>

void read_proc_stat(unsigned long *r,
		    unsigned long *w)
{
  static struct pst_vminfo buf;
  if (pstat_getvminfo(&buf, sizeof(buf), 1, 0) != -1)
  {
    (*r) = buf.psv_lreads;
    (*w) = buf.psv_lwrites;
  }
}

#endif

/* ---- Linux ---------------------------------------------------- */
#if defined(__linux__)
#define	PROC_STAT_FILE	"/proc/stat"

void read_proc_stat(unsigned long *r,
		    unsigned long *w)
{
  FILE *f;
  char *item, *arg;
  char buf[1024];
  long composite_disk_rblk = 0;
  long composite_disk_wblk = 0;

  if ((f = fopen(PROC_STAT_FILE, "r")) == NULL)
    return;

  while ((fgets(buf, sizeof(buf), f)) != NULL)
  {
    item = strtok(buf, " \t\n");
    arg = strtok(NULL, "\n");
    if (item == NULL || arg == NULL)
      continue;
    if (strcmp("disk_rblk", item) == 0)	/* kernel 2.2 format */
    {
      composite_disk_rblk += strtoul(arg, &arg, 0);
    }
    else if (strcmp("disk_wblk", item) == 0)	/* kernel 2.2 format */
    {
      composite_disk_wblk += strtoul(arg, &arg, 0);
    }
    else if (strcmp("disk_io:", item) == 0)	/* kernel 2.4 format */
    {
      int major, i_disk, n;
      unsigned long rblk, wblk, rb1, rb2, wb1, wb2;

      item = strtok(arg, " \t\n");
      while (item)
      {
	/*
	 * It may evolve - won't know till the 2.4 dust settles 
	 */
	n = sscanf(item, "(%d,%d):(%*d,%lu,%lu,%lu,%lu)",
		   &major, &i_disk, &rb1, &rb2, &wb1, &wb2);
	if (n == 6)		/* patched as of 2.4.0-test1-ac9 */
	{			/* (major,disk):(total,rio,rblk,wio,wblk) */
	  rblk = rb2;
	  wblk = wb2;
	}
	else			/* 2.3.99-pre8 to 2.4.0-testX */
	{			/* (major,disk):(rio,rblk,wio,wblk) */
	  rblk = rb1;
	  wblk = wb1;
	}
	composite_disk_rblk += rblk;
	composite_disk_wblk += wblk;
	item = strtok(NULL, " \t\n");
      }
      break;
    }
  }
  fclose(f);

  (*r) = composite_disk_rblk;
  (*w) = composite_disk_wblk;
}

#endif

static const char rcsid[] =
  "$Id: procstat.c,v 1.6 2002/10/30 16:59:35 ezix Exp $";
