#include <unistd.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Shell.h>
#include <X11/Xaw/Label.h>
#include "version.h"
#include "cursors.h"
#include "procstat.h"

typedef struct
{
  String readcolor;
  String writecolor;
  Boolean wheel;
}
XDAFResources;

XDAFResources resources;

static XrmOptionDescRec options[] = {
  {"-readcolor", ".readcolor", XrmoptionSepArg, NULL},
  {"-writecolor", ".writecolor", XrmoptionSepArg, NULL},
  {"-nowheel", ".wheel", XrmoptionNoArg, "False"},
};

#define Offset(field) (XtOffsetOf(XDAFResources, field))

static XtResource my_resources[] = {
  {"readcolor", XtCString, XtRString, sizeof(String),
   Offset(readcolor), XtRImmediate, (XtPointer) "white"}
  ,
  {"writecolor", XtCString, XtRString, sizeof(String),
   Offset(writecolor), XtRImmediate, (XtPointer) "red"}
  ,
  {"wheel", XtCBoolean, XtRBoolean, sizeof(Boolean),
   Offset(wheel), XtRImmediate, (XtPointer) TRUE}
  ,
};

Display *display;
XtAppContext appContext;

#define DISKREADING 1
#define DISKWRITING 2

int disk_activity()
{
  static unsigned long r = 0;
  static unsigned long w = 0;
  unsigned long rblk, wblk;
  int result = 0;

  read_proc_stat(&rblk, &wblk);

  if ((rblk != r) || (wblk != w))
  {
    if ((wblk - w) > (rblk - r))
      result = DISKWRITING;
    else
      result = DISKREADING;
  }

  r = rblk;
  w = wblk;

  return result;
}

void daemonize()
{
  close(0);
  close(1);
  close(2);
  setpgrp();
  if (fork())
    exit(0);
}

void usage()
{
  fprintf(stderr, "X Disk Activity Feedback - %s\n", getpackageversion());
  fprintf(stderr, "usage: xdaf [-options ...]\n");
  fprintf(stderr,
	  "    -readcolor color          color to use to represent read activity\n");
  fprintf(stderr,
	  "    -writecolor color         color to use to represent write activity\n");
  fprintf(stderr,
	  "    -nowheel                  do not show the animated wheel\n");
  fprintf(stderr, "\n");
  exit(1);
}

int main(int argc,
	 char **argv)
{
  int working = 0;
  int activity = 0;
  int zero = 0;
  Display *display;
  Widget toplevel, xdaf;

  XtSetLanguageProc(NULL, (XtLanguageProc) NULL, NULL);

  toplevel =
    XtOpenApplication(&appContext, "Xdaf", options, XtNumber(options), &argc,
		      argv, NULL, sessionShellWidgetClass, NULL,
		      (Cardinal) 0);

  if (argc != 1)
    usage();

  XtGetApplicationResources(toplevel, (XtPointer) & resources,
			    my_resources, XtNumber(my_resources),
			    NULL, (Cardinal) 0);

  daemonize();

  xdaf =
    XtCreateManagedWidget("xdaf", labelWidgetClass, toplevel, NULL, ZERO);
  XtSetMappedWhenManaged(toplevel, False);

  XtRealizeWidget(toplevel);
  display = XtDisplay(toplevel);

  XDAFCreateCursors(display);

  while (1)
  {
    XEvent event;

    if (XtAppPending(appContext))
    {
      XtAppNextEvent(appContext, &event);
      XtDispatchEvent(&event);
    }

    if (activity = disk_activity())
    {
      if (activity == DISKREADING)
	XDAFAnimateCursor(display, resources.readcolor, resources.wheel);
      else
	XDAFAnimateCursor(display, resources.writecolor, resources.wheel);
      working = 1;
    }
    else if (working)
    {
      XDAFRestoreCursor(display);
      working = 0;
    }
    usleep(10000);
  }

  return 0;
}

static char rcsid[] = "$Id: xdaf.c,v 1.22 2002/11/02 13:44:02 ezix Exp $";
